<?php

/*                                                       www.zoomstats.org
   +----------------------------------------------------------------------+
   | ZoomStats  ::  Web Traffic Analyzer                                  |
   +----------------------------------------------------------------------+
   | Copyright (c) 2002-2003 ZoomStats Group                              |
   +----------------------------------------------------------------------+
   | This file is protected by General Public License (GPL)               |
   |                                                                      |
   | GPL license does not restrict you from the usage, modification and   |
   | and redistribuition of this code. It, however, requires you to       |
   | always maintain each and every copyright (c) notice untouched.       |
   |                                                                      |
   | For full license please see the file LICENSE.TXT in this             |
   | distribution. The up-to-date version of this license is also         |
   | available online at: www.zoomstats.org/license                       |
   +----------------------------------------------------------------------+

     Project Home:
   +----------------------------------------------------------------------+
     Official Site:    http://www.zoomstats.org
     Development Area: http://sf.net/projects/zoomstats
     Latest Release:   http://sf.net/project/showfiles.php?group_id=59637
     CVS Repository:   http://sf.net/cvs/?group_id=59637
     Support & Bugs:   http://sf.net/tracker/?group_id=59637

   +----------------------------------------------------------------------+
   |                        !!! BE INTERACTIVE !!!                        |
   |                                                                      |
   | Please send us your patches, improvements, bugs, requests or just    |
   | say thanks at the links above.                                       |
   +----------------------------------------------------------------------+

     File:             $RCSfile: layer.php,v $
   +----------------------------------------------------------------------+
     Version:          $Revision: 1.3 $
     Last Updated:     $Date: 2003/06/26 19:11:22 $
     Last Author:      $Author: s-maxim $

     Maintainers:
   +----------------------------------------------------------------------+
     Maxim Maletsky    (s-maxim)
     maxim at php dot net     http://www.maxim.cx 

     Description:
   +----------------------------------------------------------------------+
     Database Abstraction Layer. 
     
     The Layer wraps up the database calls. It calles a specific API for a
     choosen database while handles no connection variables.
     
     The overall responsibility of this class is to receive the database
     setting, parameters and definitions to then pass them via a
     standarized API interface to API.
     
     API then returns to Layer the data in predefined forms and Layer
     takes care of the data by modifiying it according to calling
     script.

 */




/*   Class:            layer
   +----------------------------------------------------------------------+
     Scope:            Private
     Authors:          Maxim Maletsky (maxim at php dot net)

     Description:      ZoomStats Database Abstraction Layer.

                       Class reads queries saved in the sql dir. and
                       returns the result
   +----------------------------------------------------------------------+
*/

Class layer extends shared {

	// Private
	var $db_alias      = '';              // Query Alias
	var $db_host       = '';              // Database Host
	var $db_username   = '';              // Database Username
	var $db_password   = '';              // Database Password
	var $db_database   = '';              // Database Name
	var $db_store      = Array();         // Locations for SQL files
	var $db_query      = Array();         // Query Handler
	var $db_record     = Array();         // DB Result array
	var $db_shutdown   = False;           // Flag to surpress database connection




	/*   Method:           DB_Connect
	   +------------------------------------------------------------------+
	     Scope:            Private
	     Authors:          Maxim Maletsky (maxim at php dot net)

	     Return:           Boolean

	     API methods:      api_connect   (connect to the database API)

	     Description:      Create the connection handler for the API to
	                       Database server.

	                       The connection handler variable is never passed
	                       to this class, instead it is only used locally
	                       within API. It is only up to API to control the
	                       database connections.
	                       
	                       This function is called every time. So, databases
	                       supporting persistent connections should implement
	     +------------------------------------------------------------------+
	*/

	function db_connect() {

		// Reconnect to api
		if($this->db_shutdown !== False)
			$this->db_close();

		// Return positively on successful connection.
		Return $this->api_connect();
	}




	/*   Method:           db_query
	   +------------------------------------------------------------------+
	     Scope:            Private
	     Authors:          Maxim Maletsky (maxim at php dot net)

	     Arguments:        $alias        (int: query name)
	                       $data         (array: query name)

	     Return:           Array         (db result) or
	                       String        (db result if in 'tp_string' mode)
	                       Boolean       (for INSERT/UPDATE/non-binded PLSQL)

	     API methods:      api_escape    (escape data strings)
	                       api_execute   (execute query)
	                       api_errno     (return error number)
	                       api_error     (return error message)

	     Description:      Execute Query

	                       This function handles the data arrays, locates
	                       SQL (or other type) definitions and calls 
	                       api_execute() API function.
	                       
	                       api_execute() is supposed to actually execute
	                       the statement based on the local handlers and
	                       types. api_execute() must return True on success
	                       or Flas on failure of the execution of query.
	                       
	                       If the query fails, api_errno() and api_error()
	                       are invoced to return the failure details.
	                       
	                       api_execute() proceeds (if successul) till the
	                       returning whether the DB result or statement
	                       LAST_INSERT_ID / AFFECTED_ROWS equivalents,
	                       depending on statement type controlled via
	                       switch (value originates in DB storage files).
	   +------------------------------------------------------------------+
	*/


	function query($alias = '', $data = '') {
		Return $this->db_query($alias, $data);
	}

	function db_query($alias = '', $data = '') {

		// Connect to API
		if(!$this->db_connect())
			Return False;

		// Clean session
		$this->db_free();

		// Store Alias
		$this->db_alias = $alias;

		// Import variabiles
		if(is_array($data))
			$this->db_data = $data;

		// Escape strings by API-specific methods
		foreach($this->db_data as $key=>$val)
			$$key = $this->api_escape($val);

		$table_prefix = $this->table_prefix;

		// Include SQL storage files
		foreach($this->db_store as $section) {
			if(file_exists($section)) {
				@include($section);
				$loaded = True;
			}
			else
				@$store .= "\n$section";
		}

		// Check for file existence
		// if not found
		if(!isset($loaded))
			$this->error('db_no_store', FATAL, Array(), $this->api_name, __FILE__, __LINE__);

		// if can't load
		elseif(@$store)
			$this->error('db_cant_load_store', FATAL, Array('store' => "$store\n"), $this->api_name, __FILE__, __LINE__);

		// Check existence of query
		$this->db_query = isset($query)? $query : $this->error('db_no_query', FATAL, Array('alias' => $this->db_alias, 'store'=>implode("\n", $this->db_store) . "\n"), $this->api_name, __FILE__, __LINE__);

		// Clean trailing semicolons ';' in SQL 
		$this->db_query['sql']  = isset($this->db_query['sql'])? preg_replace("/;?\s*$/s", '', $this->db_query['sql']) : '';

		// Execute Query
		if(!$this->api_execute()) {
			// on failure, create error message
			$line               = __LINE__;
			$file               = __FILE__; 
			$details['alias']   = $this->db_alias;
			$details['errno']   = $this->api_errno();
			$details['error']   = $this->api_name . ' said: ' . $this->api_error();

			// we print SQL by default. To protect it use [quiet] in SQL files
			if(!$this->db_quiet and !isset($this->db_query['quiet']))
				$details['sql'] = "\nSQL query:\n{$this->db_query['sql']}";

			// call error handler
			$this->error('db_query_error', FATAL, $details, $this->api_name, $file, $line);
		}

		// Switch between statement types:
		switch(strtolower($this->db_query['type'])) {

			case 'select':
				Return $this->db_result();
				break;

			case 'update':
				Return True;
				break;

			case 'insert':
				Return True;
				break;

			case 'delete':
				Return True;
				break;

			default:
				Return False;
		}
	}




	/*   Method:           db_result
	   +------------------------------------------------------------------+
	     Scope:            Private

	     Authors:          Maxim Maletsky (maxim at php dot net)
	                       Rune Bromer (bromer at diku dot dk)

	     Return:           array | string (depending on 'TO_STRING' flag)

	     API methods:      api_rows      (return number of rows)
	                       api_cols      (return number of columns)
	                       api_next_record (return next record (row))

	     Description:      Return results in the user-predefined form.

	                       db_result makes use of several API functions.
	                       All the array/db specific configurations are
	                       globally executed here.
	                       
	                       Method expects api_next_record() to produce
	                       the variable $this->db_record in the following 
	                       form - Array([,column record...]).
	                       
	                       $this->db_record must start from the first row
	                       and continue till the last row is pulled. If no
	                       rows are more to pull, api_next_record() must
	                       returm FALSE.
	   +------------------------------------------------------------------+
	*/

	function db_result() {

		// Return Result if defined
		if(!empty($this->db_result))
			Return $this->db_result;

		// Number of Rows
		$this->db_rows     = $this->api_rows();

		// Number of Columns
		$this->db_cols     = $this->api_cols();

		// Limit Rows
		if(is_int($this->db_limit) and $this->db_limit > 0)
			$this->db_rows = $this->db_limit;

		// Check Personalized Key
		if(strlen($this->db_key))
			$custom_key = $this->db_key;

		else if(isset($this->db_query['key']) and strlen($this->db_query['key'])>0)
			$custom_key = $this->db_query['key'];

		// Compose array
		$r = 0;
		while($r<$this->db_rows and $this->api_next_record($r)) {

			// Add key
			$key = isset($custom_key)? $this->db_record[$custom_key] : $r;

			// Loop values
			foreach($this->db_record as $col=>$val) {
				$this->db_result[$key][$col] = $val;
			}

			// increment row count
			$r++;
		}

		// Prepare 'to_string' mode
		if(isset($this->db_query['to_string'])) {

			// custom key cannot be set for 'to_string' mode
			if(isset($custom_key))
				$this->error('db_cant_convert_to_string_key', FATAL, Array('key'=>$this->db_key, 'alias'=>$this->db_alias), $this->api_name, __FILE__, __LINE__);

			// DB result must be only one row
			elseif(sizeof($this->db_result)!=1)
				$this->error('db_cant_convert_to_string_arr', FATAL, Array('num'=>sizeof($this->db_result), 'alias'=>$this->db_alias), $this->api_name, __FILE__, __LINE__);

			// the first row of DB result must have only one field catched
			elseif(sizeof($this->db_result[0])!=1)
				$this->error('db_cant_convert_to_string_res', FATAL, Array('num'=>sizeof($this->db_result[0]), 'alias'=>$this->db_alias), $this->api_name, __FILE__, __LINE__);

			// replace result array by the string value of its only field contained. Name gets lost
			else
				$this->db_result = current($this->db_result[0]);
		}

		// Return Results
		Return $this->db_result;
	}



	/*   Method:           db_free
	   +------------------------------------------------------------------+
	     Scope:            Private

	     Authors:          Maxim Maletsky (maxim at php dot net)
	                       Rune Bromer (bromer at diku dot dk)

	     Return:           Boolean

	     API methods:      api_free      (cleans up local API variables)

	     Description:      Free-up Database

	                       Cleans up all per-session variable, both public
	                       and private
	   +------------------------------------------------------------------+
	*/

	function db_free() {
		
		// Clean Session Data
		$this->db_query = $this->db_result = $this->db_record = Array();

		// Reset flags
		$this->db_key = $this->db_alias = $this->db_limit = '';

		// Clean API's private variables
		$this->api_free();

		// Clean API's private variables
		$this->error_id = '';

		Return True;
	}



	/*   Method:           db_close
	   +------------------------------------------------------------------+
	     Scope:            Private

	     Authors:          Maxim Maletsky (maxim at php dot net)

	     Return:           Boolean

	     API methods:      api_free      (cleans up local API variables)

	     Description:      Close Database Connection

	                       Calls API-Specific function to close the
	                       database connection.

	                       returns always True
	   +------------------------------------------------------------------+
	*/

	function db_close() {

		// Close database connection
		$this->api_close();

		Return True;
	}
}

?>