<?php

/*                                                       www.zoomstats.org
   +----------------------------------------------------------------------+
   | ZoomStats  ::  Web Traffic Analyzer                                  |
   +----------------------------------------------------------------------+
   | Copyright (c) 2002-2003 ZoomStats Group                              |
   +----------------------------------------------------------------------+
   | This file is protected by General Public License (GPL)               |
   |                                                                      |
   | GPL license does not restrict you from the usage, modification and   |
   | and redistribuition of this code. It, however, requires you to       |
   | always maintain each and every copyright (c) notice untouched.       |
   |                                                                      |
   | For full license please see the file LICENSE.TXT in this             |
   | distribution. The up-to-date version of this license is also         |
   | available online at: www.zoomstats.org/license                       |
   +----------------------------------------------------------------------+

     Project Home:
   +----------------------------------------------------------------------+
     Official Site:    http://www.zoomstats.org
     Development Area: http://sf.net/projects/zoomstats
     Latest Release:   http://sf.net/project/showfiles.php?group_id=59637
     CVS Repository:   http://sf.net/cvs/?group_id=59637
     Support & Bugs:   http://sf.net/tracker/?group_id=59637

   +----------------------------------------------------------------------+
   |                        !!! BE INTERACTIVE !!!                        |
   |                                                                      |
   | Please send us your patches, improvements, bugs, requests or just    |
   | say thanks at the links above.                                       |
   +----------------------------------------------------------------------+

     File:             $RCSfile: mysql.php,v $
   +----------------------------------------------------------------------+
     Version:          $Revision: 1.4 $
     Last Updated:     $Date: 2003/06/26 19:10:21 $
     Last Author:      $Author: s-maxim $

     Maintainers:
   +----------------------------------------------------------------------+
     Maxim Maletsky    (s-maxim)
     maxim at php dot net     http://www.maxim.cx

     Rune Bromer       (bromer)
     bromer at diku dot dk

     Description:
   +----------------------------------------------------------------------+
     MySQL API

     API is the back-level connection resource handler. It can be expanded
     into any amount of data handling procedures, however, all the public
     must preserve their names and return structure.
     
     API is ONLY invoked by Layer.php, thus is may freely use the
     following (public) parent variables:
     
     $db_host       = '';              // Database Host
     $db_username   = '';              // Database Username
     $db_password   = '';              // Database Password
     $db_database   = '';              // Database Name
     $db_query      = Array();         // Query Handler
     $db_record     = Array();         // DB Result array
     
     It is possible to modify these variables per each session, however
     by doing so you risk incompatibility problems should something change
     with a parent class (Layer). It is reccomended to note such chanegs
     to Layer's developers.

 */



/*   Class:            api (MySQL)
   +----------------------------------------------------------------------+
     Scope:            Private
     Authors:          Maxim Maletsky (maxim at php dot net)
                       Rune Bromer (bromer at diku dot dk)

     Description:      MySQL database abstraction Layer.

                       API is the back-level connection resource handler.
                       It can be expanded into any amount of data handling 
                       procedures, however, all the public must preserve 
                       their names and return structure.
   +----------------------------------------------------------------------+
*/

// Load Shared DB
include_once "{$GLOBALS['lib']['db']['path']}/shared.php";

Class db extends layer {

	// Public
	var $api_name       = 'MySQL';         // Api literal name

	// Private
	var $api_conn       = False;           // Connection Handler
	var $api_stmt       = False;           // Statement Handler



	/*   Method:           api_connect
	   +------------------------------------------------------------------+
	     Scope:            Public (invoked by Layer.php)
	     Authors:          Rune Bromer (bromer at diku dot dk)
	                       Maxim Maletsky (maxim at php dot net)

	     Return:           Boolean (True on success, False on failure.
	                       In case of false a fatal error is called)

	     Description:      This is the standard method to open the
	                       database connection. api_connect() is always
	                       called. It stays up to api_connect() to decide
	                       on how to create and maintain the connection
	                       link.
	                       
	                       It doesn't matter how this method builds the
	                       connection and how many things it need to
	                       pre-process.
	                       
	                       The ultimate goal for api_Connect() method it to
	                       create a private variable for the eventual reuse
	                       while manipulating the database.
	   +------------------------------------------------------------------+
	*/

	function api_connect() {
		
		// if API is already connected, Exit
		if($this->api_conn)
			Return True;

		// Connect to MySQL Server
		if(!$this->api_conn = @mysql_pconnect($this->db_host, $this->db_username, $this->db_password)) {

			#echo $this->api_errno();

			switch($this->api_errno()) {
				case 2005:
					Return $this->error('db_no_host', FATAL, Array('type' => $this->api_name, 'host' => $this->db_host, 'message' => $this->api_error()), $this->api_name, __FILE__, __LINE__);
					echo "Hell Yeah!";
					break;

				default:
					Return $this->error('db_cant_connect', FATAL, Array('type' => $this->api_name, 'host' => $this->db_host, 'user' => $this->db_username, 'message' => $this->api_error()), $this->api_name, __FILE__, __LINE__);
					break;
			}

			Return False;
		}

		// Hook onto a MySQL database
		if(!mysql_select_db($this->db_database, $this->api_conn)) {
			Return $this->error('db_cant_select', FATAL, Array('db' => $this->db_database, 'message' => $this->api_error()), $this->api_name, __FILE__, __LINE__);
			Return False;
		}

		// Return positively on successful connection
		Return True;
	}




	/*   Method:           api_execute
	   +------------------------------------------------------------------+
	     Scope:            Public (invoked by Layer.php)
	     Authors:          Maxim Maletsky (maxim at php dot net)

	     Return:           Boolean (True on success, False on failure.
	                       In case of False a fatal error is called)

	     Description:      This method controls the database query
	                       execution.
	                       
	                       Regardless of the procedures this function needs
	                       to create, it's only goal it to execute a query
	                       and prepare a local object for eventual data
	                       pulling
	                       
	                       This function MUST return True on success or
	                       False on failure. A False return will change
	                       the behavior of Layer.php
	   +------------------------------------------------------------------+
	*/

	function api_execute() {
		Return ($this->api_stmt = @mysql_query($this->db_query['sql'], $this->api_conn));
	}




	/*   Method:           api_next_record
	   +------------------------------------------------------------------+
	     Scope:            Public (invoked by Layer.php)
	     Authors:          Maxim Maletsky (maxim at php dot net)

	     Return:           Boolean (True on success, False on failure.
	                       In case of False a fatal error is called)

	     Description:      This method controls the record retrieval
	                       
	                       This method is called in the loop by Layer for
	                       as long as it returns False (which is what
	                       api_next_record() should return after the last
	                       row was retrieved.
	                       
	                       Method's primary goal is to assign the array 
	                       result for every subsequent row to
	                       $this->db_record variable.
	                       
	                       api_next_record() needs to producethe variable
	                       $this->db_record in the following form :
	                       Array([,column record...]).
	                       
	                       $this->db_record must start from the first row
	                       and continue till the last row is pulled. If no
	                       rows are more to pull, api_next_record() must
	                       returm FALSE.
	   +------------------------------------------------------------------+
	*/

	function api_next_record() {
		Return ($this->db_record = @mysql_fetch_array($this->api_stmt, MYSQL_ASSOC));
	}




	/*   Method:           api_escape
	   +------------------------------------------------------------------+
	     Scope:            Public (invoked by Layer.php)
	     Authors:          Maxim Maletsky (maxim at php dot net)

	     arguments:        $var (string)
	     Return:           string

	     Description:      Escapes the string according to the
	                       database-specific string syntaxing.
	                       
	                       api_escape receives an array and modifies it
	                       to create a SQL-SAFE string to be directly
	                       included within SQL commands.
	   +------------------------------------------------------------------+
	*/

	function api_escape($var) {

		if(is_array($var)) {
			foreach($var as $k=>$v)
				$var[$k] = $this->api_escape($v);

			Return $var;
		}
		
		Return @mysql_escape_string($var);
	}




	/*   Method:           api_errno
	   +------------------------------------------------------------------+
	     Scope:            Public (invoked by Layer.php)
	     Authors:          Maxim Maletsky (maxim at php dot net)

	     Return:           integer

	     Description:      Return the last error number.
	                       
	                       This method is called on a db execution failure
	                       to prepare a more personalized error message.
	                       
	                       It doesn't matter the logic a number is made by,
	                       but it must be something easily translated if
	                       found from logs
	   +------------------------------------------------------------------+
	*/

	function api_errno() {
		Return @mysql_errno();
	}




	/*   Method:           api_error
	   +------------------------------------------------------------------+
	     Scope:            Public (invoked by Layer.php)
	     Authors:          Maxim Maletsky (maxim at php dot net)

	     Return:           String

	     Description:      Return the last error message.
	                       
	                       This method is called on a db execution failure
	                       to prepare a more personalized error message.
	                       
	                       It doesn't matter the logic the message is made
	                       by, but it must be something easily translated
	                       if found from logs
	   +------------------------------------------------------------------+
	*/

	function api_error() {
		Return @mysql_error();
	}




	/*   Method:           api_rows
	   +------------------------------------------------------------------+
	     Scope:            Public (invoked by Layer.php)
	     Authors:          Maxim Maletsky (maxim at php dot net)

	     Return:           integer

	     Description:      Returns the number of records found
	                       
	                       This function is always called after api_execute
	                       Its only goal is to know what amount of records
	                       a query has retrieved
	   +------------------------------------------------------------------+
	*/

	function api_rows() {
		$rows = @mysql_num_rows($this->api_stmt);
		Return $rows? $rows : 0;
	}




	/*   Method:           api_cols
	   +------------------------------------------------------------------+
	     Scope:            Public (invoked by Layer.php)
	     Authors:          Maxim Maletsky (maxim at php dot net)

	     Return:           integer

	     Description:      Returns the number of columns/fields returned
	                       
	                       This function is always called after api_execute
	                       Its only goal is to know what amount of fields
	                       a query has retrieved.
	                       
	                       ALL RETURN ARRAYS MUST BE SIMMETRICAL!
	   +------------------------------------------------------------------+
	*/

	function api_cols() {
		$cols = @mysql_num_fields($this->api_stmt);
		Return $cols? $cols : 0;
	}




	/*   Method:           api_last_id
	   +------------------------------------------------------------------+
	     Scope:            Public (invoked by Layer.php)

	     Authors:          Rune Bromer (bromer at diku dot dk)
	                       Maxim Maletsky (maxim at php dot net)

	     Return:           mixed

	     Description:      Returns the last inserted ID.
	                       
	                       Hopefully, this function will never be called as
	                       because of this logic being vulnerable. However,
	                       what it should do, is to tell the LAST ID
	                       inserted by the current connection.
	                       
	                       if none were made, a NULL should be returned
	   +------------------------------------------------------------------+
	*/

	function api_last_id() {
		$id = @mysql_insert_id($this->api_conn);
		Return strlen($id)? $id : Null;
	}




	/*   Method:           api_affected_rows
	   +------------------------------------------------------------------+
	     Scope:            Public (invoked by Layer.php)

	     Authors:          Maxim Maletsky (maxim at php dot net)

	     Return:           int

	     Description:      Returns the number of affected rows by an INSERT
	                       or an UPDATE.
	                       
	                       This MUST be an integer
	                       
	                       If none were made, '0' should be returned
	   +------------------------------------------------------------------+
	*/

	function api_affected_rows() {
		return @mysql_affected_rows($this->api_conn);
	}



	/*   Method:           api_free
	   +------------------------------------------------------------------+
	     Scope:            Public (invoked by Layer.php)
	     Authors:          Maxim Maletsky (maxim at php dot net)

	     Return:           Boolean

	     Description:      Frees up every possible private variables that
	                       are not reusable for the further calls within
	                       this session (connection).
	   +------------------------------------------------------------------+
	*/

	function api_free() {

		// free up the memory
		@mysql_free_result($this->api_stmt);

		$this->api_stmt = False;
		Return True;
	}




	/*   Method:           api_close
	   +------------------------------------------------------------------+
	     Scope:            Public (invoked by Layer.php)
	     Authors:          Maxim Maletsky (maxim at php dot net)

	     Return:           Boolean

	     Description:      Closes any active database connections and
	                       resets its variable pointers.
	                       
	                       Invoking api_free() method is a good practice.
	   +------------------------------------------------------------------+
	*/

	function api_close() {
		$this->api_free();
		@mysql_close($this->api_conn);
		Return $this->api_conn = False;
	}
}

?>