<?php

/*                                                       www.zoomstats.org
   +----------------------------------------------------------------------+
   | ZoomStats  ::  Web Traffic Analyzer                                  |
   +----------------------------------------------------------------------+
   | Copyright (c) 2002-2003 ZoomStats Group                              |
   +----------------------------------------------------------------------+
   | This file is protected by General Public License (GPL)               |
   |                                                                      |
   | GPL license does not restrict you from the usage, modification and   |
   | and redistribuition of this code. It, however, requires you to       |
   | always maintain each and every copyright (c) notice untouched.       |
   |                                                                      |
   | For full license please see the file LICENSE.TXT in this             |
   | distribution. The up-to-date version of this license is also         |
   | available online at: www.zoomstats.org/license                       |
   +----------------------------------------------------------------------+

     Project Home:
   +----------------------------------------------------------------------+
     Official Site:    http://www.zoomstats.org
     Development Area: http://sf.net/projects/zoomstats
     Latest Release:   http://sf.net/project/showfiles.php?group_id=59637
     CVS Repository:   http://sf.net/cvs/?group_id=59637
     Support & Bugs:   http://sf.net/tracker/?group_id=59637

   +----------------------------------------------------------------------+
   |                        !!! BE INTERACTIVE !!!                        |
   |                                                                      |
   | Please send us your patches, improvements, bugs, requests or just    |
   | say thanks at the links above.                                       |
   +----------------------------------------------------------------------+

     File:             $RCSfile: shared.php,v $
   +----------------------------------------------------------------------+
     Version:          $Revision: 1.5 $
     Last Updated:     $Date: 2003/06/26 19:10:04 $
     Last Author:      $Author: s-maxim $

     Maintainers:
   +----------------------------------------------------------------------+
     Maxim Maletsky    (s-maxim)
     maxim at php dot net     http://www.maxim.cx 

     Description:
   +----------------------------------------------------------------------+
     Shared Objects Layer.

 */




/*   Class:            shared
   +----------------------------------------------------------------------+
     Scope:            Private
     Authors:          Maxim Maletsky (maxim at php dot net)

     Description:      Shared Object for Database Layer. Contains the
                       methods to be shared between class inheritation
   +----------------------------------------------------------------------+
*/

// Load Shared DB
include_once "{$GLOBALS['lib']['db']['path']}/layer.php";

Class shared {
	
	// Global Variables
	var $parent;                          // Reference to the parent calling class
	var $conf;                            // Local configuration variable
	var $api_type;                        // Current API type
	var $section;                         // Current section
	var $error;                           // Error object
	var $error_quiet = False;             // Surpress the errors?
	var $error_id;                        // Last error received
	var $error_data = Array();            // Last error data
	var $stop_transaction = False;        // Stop transaction flag
	var $stop_execution = False;          // Stop execution

	// Shared API variables
	var $db_data       = Array();         // Data passed to queries
	var $db_result     = Array();         // Query results
	var $db_key        = '';              // Column name desired to be the Result array key.
	var $db_limit      = 0;               // Limit result to N rows.
	var $db_cols       = 0;               // Number of Comumns
	var $db_rows       = 0;               // Number of Rows
	var $db_rows       = 0;               // Number of Rows

	var $db_quiet      = False;           // Whether should DB errors messages dysplay the entire SQL
	                                      // Controlled also by ['quiet'] parameter within SQL.
	                                      // Used primarely for testing

	var $user_obj;                        // User object
	var $keep_awake    = False;           // Flag to avoid timed-out responces
	var $first_pack    = False;           // Flag to recognize the first pack so we insert instead of updating

	var $table_prefix  = '';              // Prefix for use with the tables



	/*   Method:           configure
	   +------------------------------------------------------------------+
	     Scope:            Private
	     Author:           Maxim Maletsky (maxim at php dot net)

	     Arguments:        $parent (reference to calling object)
	     Return:           Boolean

	     Description:      Main constructor for Shared Object.

	                       Builds and executes configuration settings.
	   +------------------------------------------------------------------+
	*/

	function configure($parent) {

		// Reference Parent ZoomStats Class
		$this->conf             = &$parent->conf;
		$this->conf['settings'] = Array();

		// Configure Connection Variables
		$this->api_configure();

		// Unpack Configurations Variables
		$this->unpack();

		// temporary mask for testing inslaller
		if(isset($_GET['install'])) {

			// Control if this is the first run
			if(!isset($this->conf['settings']['version'])) {
				// Install
				include_once $this->conf['path']['lib'] . 'installer/install.php';
				new install($this);
				Exit;
			}

			// control versions change
			elseif($this->vcompare($this->conf['version'], $this->conf['settings']['version'])) {
				// Upgrade
				include_once $this->conf['path']['lib'] . 'installer/upgrade.php';
				new upgrade($this);
				Exit;
			}
		}

		// Register this for shutdown
		register_shutdown_function(Array($this, 'shutdown'));

		// Authenticate User
		if(!$this->authenticate()) {
			// Wait a bit against hacker machines
			if(!$this->keep_awake)
				sleep(5);

			Return Exit;
		}
		Return True;
	}




	/*   Method            shutdown
	   +------------------------------------------------------------------+
	     Scope:            Private

	     Author:           Maxim Maletsky (maxim at php dot net)

	     Description:     Executes less commands at the end of process
	   +------------------------------------------------------------------+
	*/

	function shutdown() {
		// Connection needs to be closed before reopening it
		$this->db_shutdown = True;

		// Close database connection
		$this->db_close();
	}



	/*   Method:           api_configure
	   +------------------------------------------------------------------+
	     Scope:            Private
	     Author:           Maxim Maletsky (maxim at php dot net)

	     Arguments:        
	     Return:           Boolean

	     Description:      Main constructor for API.

	                       Builds and executes configuration settings.
	   +------------------------------------------------------------------+
	*/

	function api_configure() {

		// Buld API Type
		if(!empty($this->parent['api']['type']))
			$this->api_type        = $this->parent['api']['type'];

		// Locate SQL files
		$this->db_store = Array(
			 'shared'   => $this->conf['path']['db'] . 'clai.sql.php'
			,'section'  => $this->conf['path']['db'] . 'sql/' . $this->conf['section'] . '.sql.php'
		);

		// Set API Hostname
		if(!empty($this->conf['api']['host']))
			$this->db_host         = $this->conf['api']['host'];

		// Set API Username
		if(!empty($this->conf['api']['username']))
			$this->db_username     = $this->conf['api']['username'];

		// Set API Password
		if(!empty($this->conf['api']['password']))
			$this->db_password     = $this->conf['api']['password'];

		// Set API Database Name
		if(!empty($this->conf['api']['database']))
			$this->db_database     = $this->conf['api']['database'];

		Return True;
	}

	/*   Method:           error
	   +------------------------------------------------------------------+
	     Scope:            Private
	     Author:           Maxim Maletsky (maxim at php dot net)

	     Arguments:        
	     Return:           Object

	     Description:      Calls errors Class on first error occured
	   +------------------------------------------------------------------+
	*/

	function error($error=null, $type=True, $data=null, $zone='', $file='[unspecified]', $line=0) {

		if(!$this->error_quiet){
			echo "<B>$error</B> on line $line ($file)<br>Details:";
			r($data);
		}

		$this->error_id   = $error;
		$this->error_data = $data;

		Return ($type == FATAL)? False : True;
	}


}

?>